local _, _, REVISION = string.find("$Revision: 17278 $", "(%d+)")

--[[
-- lua 5.1 check
if(not loadstring("return function(...) return ... end")  and true or false) then
	string.gmatch = string.gfind;
    math.fmod = math.mod;    
    table.maxn = table.getn;
end
--]]

local lang = AceLibrary("AceLocale-2.2"):new("SchoolTeacher")

lang:RegisterTranslations("enUS", function() return {
	["Teacher"] = true,

	["Rebuild"] = true,
	["Rebuild teacher data."] = true,
    
	["Advanced Mode"] = true,
	["Toggles advanced teacher options visibility."] = true,    
} end)

SchoolTeacher = School:NewModule("Teacher")

SchoolTeacher.defaults = {
	advancedMode = false,
}

SchoolTeacher.options = {
    rebuild = {
               type = 'execute',
               name = lang["Rebuild"],
               desc = lang["Rebuild teacher data."],
               func = function() SchoolTeacher:Rebuild() end,
               order = 110,
    },

	advancedMode = {
		type = "toggle",
		name = lang["Advanced Mode"],
		desc = lang["Toggles advanced teacher options visibility."],
		get = function() return SchoolTeacher:ToggleAdvancedMode() end,
		set = function(v) SchoolTeacher:ToggleAdvancedMode(v) end,
		order = 120,
	},
}

SchoolTeacher:SetDebugging(true)
SchoolTeacher:SetDebugLevel(3)

SchoolTeacher.revision = tonumber(REVISION)
SchoolTeacher.triggers = true

function SchoolTeacher:OnInitialize()
end

function SchoolTeacher:OnEnable()
    self:RegisterEvent("TRAINER_SHOW")
    self:RegisterEvent("TRAINER_CLOSED")
    
    self.class = self.core.classid
end

function SchoolTeacher:OnDisable()
end

function SchoolTeacher:GetData()
    local data = {}

    if (not self.db.profile.spells) then self:LogTrainerInfo() end
    data.spells = self.db.profile.spells
    
    data.trainers = self.db.profile.trainers
    
    return data
end

function SchoolTeacher:ToggleAdvancedMode()
	if type(value) == "nil" then
		return self.db.profile.advancedMode
	end

	self.db.profile.advancedMode = value
end

function SchoolTeacher:TRAINER_SHOW()
    self.DoneAtTrainer = false
end

function SchoolTeacher:TRAINER_CLOSED()
    if (self.DoneAtTrainer == false) then
        self.DoneAtTrainer = true
        self:LogTrainerInfo()
    end
end

function SchoolTeacher:Rebuild()
    return self:GetData()
end

function SchoolTeacher:LogTrainerInfo()
    if (IsTradeskillTrainer() == true) then
        self:LogTradeskillTrainerInfo()
    else
        self:LogClassTrainerInfo()
    end
    
    self:TriggerEvent("School_Teacher_Changed")
end

function SchoolTeacher:LogTradeskillTrainerInfo()
    self:LevelDebug(3, "LogTradeskillTrainerInfo")
end

function SchoolTeacher:LogClassTrainerInfo()
    self:Debug("LogClassTrainerInfo")
    local spells = {}
    local trainers = {}

    if (self.db.profile.spells ~= nil) then spells = self.db.profile.spells end
    if (self.db.profile.trainers ~= nil) then trainers = self.db.profile.trainers end
    
    local npc = {}
    npc.name = UnitName("npc")
    npc.zone = GetZoneText()
    npc.subzone = GetSubZoneText()
   
    local filter = {}
    filter.available = GetTrainerServiceTypeFilter("available")
    filter.unavailable = GetTrainerServiceTypeFilter("unavailable")
    filter.used = GetTrainerServiceTypeFilter("used")
    
    SetTrainerServiceTypeFilter("available", 1)
    SetTrainerServiceTypeFilter("unavailable", 1)
    SetTrainerServiceTypeFilter("used", 1)
    
    ExpandTrainerSkillLine(0)
    
    local numTrainerServices = GetNumTrainerServices()
    
    if (not numTrainerServices or (numTrainerServices < 1)) then
        return
    end
    
    local category = nil
    for i = 1, numTrainerServices do
        local service = {}
        
        service.name, service.subtext, service.type, service.expanded = GetTrainerServiceInfo(i)

        if (not service.name or not service.type) then return end
        if (service.type == "header") then
            category = service.name
        else
            if (category ~= nil) then service.category = category end
            if (not service.subtext) or (string.len(service.subtext) < 1) then service.subtext = "None" end
            service.fullname = service.name
            
            if (service.subtext ~= "None") then
                -- Check for "RANK"
                service.fullname = service.fullname .. " (" .. service.subtext .. ")"
            end
            
            service.cost = {}
            -- Money, Character Point Cost 1, Character Point Cost 2
            service.cost.money, service.cost.cost1, service.cost.cost2 = GetTrainerServiceCost(i)

            service.icon = GetTrainerServiceIcon(i)
            service.level = GetTrainerServiceLevelReq(i)
        
            local req = {}
            req.skill, req.rank, req.hasReq = GetTrainerServiceSkillReq(i)
            if (req.skill) then service.req = req end
            
            local numRequirements = GetTrainerServiceNumAbilityReq(i);
            if (numRequirements > 0) then
                service.ability = {}
                for k = 1, numRequirements do
                    service.ability[k] = {}
                    service.ability[k].skill, service.ability[k].req = GetTrainerServiceAbilityReq(i, k)
                end
            end

            -- Fix wrapped descriptions
            -- service.description = GetTrainerServiceDescription(i)
            
            service.learnspell, service.petlearnspell = IsTrainerServiceLearnSpell(i)
            
            
            if (not spells[service.category]) then spells[service.category] = {} end
            if (not spells[service.category][service.name]) then spells[service.category][service.name] = {} end
            if (not spells[service.category][service.name][service.subtext]) then spells[service.category][service.name][service.subtext] = {} end
            spells[service.category][service.name][service.subtext] = service
            
            if (not trainers[service.name]) then trainers[service.name] = {} end
            if (not trainers[service.name][service.subtext]) then trainers[service.name][service.subtext] = {} end
            if (not trainers[service.name][service.subtext][npc.name]) then trainers[service.name][service.subtext][npc.name] = npc end
        end
    end

    self.db.profile.spells = spells
    self.db.profile.trainers = trainers
end

function SchoolTeacher:LogTalentInfo()
    -- self:LevelDebug(3, "LogTalentInfo")
end

function SchoolTeacher:BuildSkills()
    local skills = {}
end




